package jp.deadend.noname.lastapp;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.util.Log;

public class ExcludeListActivity extends ListActivity {
	private class AppInfo implements Comparable<AppInfo> {
		Drawable icon;
		String appName;
		String packageName;
		String className;
		boolean isChecked;

		public AppInfo(Drawable ic, String na, String pn, String cn, boolean ck) {
			icon = ic;
			appName = na;
			packageName = pn;
			className = cn;
			isChecked = ck;
		}

		public int compareTo(AppInfo other) {
			return this.appName.compareTo(other.appName);
		}
	}
	static final String KEY_EXCLUDED_APP = "PrefExcludedApp";

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		LoadAppListTask task = new LoadAppListTask();
		task.execute();
	}

	@Override
	protected void onPause() {
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
		ExcludeListAdapter adapter = (ExcludeListAdapter)getListAdapter();
		StringBuilder sb = new StringBuilder();
		for (int i=0; i<adapter.getCount(); i++) {
			AppInfo item = adapter.getItem(i);
			if (!item.isChecked) {
				sb.append(item.packageName + "," + item.className + ":");
			}
		}

		sp.edit().putString(KEY_EXCLUDED_APP, sb.toString()).commit();

		super.onPause();
	}

	@Override
	protected void onListItemClick (ListView l, View v, int position, long id) {
		CheckBox cb = (CheckBox)v.findViewById(R.id.CheckBox);
		Boolean new_val = !cb.isChecked();
		cb.setChecked(new_val);

		((AppInfo)l.getItemAtPosition(position)).isChecked = new_val;
	}

	private class ExcludeListAdapter extends ArrayAdapter<AppInfo> {
		private LayoutInflater mInflater;

		class ViewHolder {
			ImageView iconView;
			TextView appNameView;
			CheckBox checkBox;
		}

		public ExcludeListAdapter(Context context, List<AppInfo> items) {
			super(context, 0, items);
			mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    	}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			ViewHolder holder;

			View view = convertView;
			if (view == null) {
				view = mInflater.inflate(R.layout.list_item, null);

				ImageView imv = (ImageView)view.findViewById(R.id.IconView);
				TextView tv = (TextView)view.findViewById(R.id.AppNameView);
				CheckBox cb = (CheckBox)view.findViewById(R.id.CheckBox);

				holder = new ViewHolder();
				holder.iconView = imv;
				holder.appNameView = tv;
				holder.checkBox = cb;

				view.setTag(holder);
    		} else {
				holder = (ViewHolder)view.getTag();
    		}

    		AppInfo item = getItem(position);
    		holder.iconView.setImageDrawable(item.icon);
    		holder.appNameView.setText(item.appName);
			holder.checkBox.setChecked(item.isChecked);

    		return view;
    	}
    }

	private class LoadAppListTask extends AsyncTask<Void, Void, List<AppInfo>> {
		ProgressDialog mProgressDialog;

		@Override
		protected void onPreExecute() {
			mProgressDialog = ProgressDialog.show(ExcludeListActivity.this, "",   getResources().getString(R.string.label_loading), true);
		}

		@Override
		protected List<AppInfo> doInBackground(Void ... params) {
			PackageManager pm = getPackageManager();
			Intent intent = new Intent(Intent.ACTION_MAIN, null);
			intent.addCategory(Intent.CATEGORY_LAUNCHER);
			List<ResolveInfo> appList = pm.queryIntentActivities(intent, 0);

			List<AppInfo> result = new ArrayList<AppInfo>(appList.size());
			if (appList.size() > 0) {
				for (ResolveInfo ri : appList) {
					String appName = ri.loadLabel(pm).toString();
					if (appName.equals(getResources().getString(R.string.app_name))) {
						continue;
					}
					ActivityInfo ai = ri.activityInfo;
					result.add(new AppInfo(ri.loadIcon(pm), appName, ai.packageName, ai.name, true));
				}
				Collections.sort(result);
			}

			SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
			String prefStr = sp.getString(KEY_EXCLUDED_APP, "");
			String[] apps = prefStr.split(":");

			for (AppInfo item : result) {
				for (String str : apps) {
					String[] names = str.split(",");
					if (item.packageName.equals(names[0]) && item.className.equals(names[1])) {
						item.isChecked = false;
						break;
					}
				}
			}

			return result;
		}

		@Override
		protected void onPostExecute(List<AppInfo> result) {
			mProgressDialog.dismiss();

			setListAdapter(new ExcludeListAdapter(ExcludeListActivity.this, result));
		}

		@Override
		protected void onCancelled() {
			if (mProgressDialog.isShowing()) {
				mProgressDialog.dismiss();
			}
		}
	}
}